@@ -529,6 +529,7 @@ char * pkg_formatted_field(pkg_t *pkg, const char *field )
      static size_t LINE_LEN = 128;
      char * temp = (char *)malloc(1);
      int len = 0;
+     int flag_provide_false = 0;
 
 /*
   Pigi: After some discussion with Florian we decided to modify the full procedure in 
@@ -702,7 +703,7 @@ char * pkg_formatted_field(pkg_t *pkg, const char *field )
 	          return NULL;
                }
                temp[0]='\0';
-               snprintf(temp, 29, "Installed-Time: %ul\n", pkg->installed_time);
+               snprintf(temp, 29, "Installed-Time: %lu\n", pkg->installed_time);
 	  }
      }
 	  break;
@@ -761,23 +762,35 @@ char * pkg_formatted_field(pkg_t *pkg, const char *field )
 	       int i;
 
 	       if (pkg->provides_count) {
-                    char provstr[LINE_LEN];
-                    len = 15;
-		    for(i = 0; i < pkg->provides_count; i++) {
-                        len = len + (strlen(pkg->provides_str[i])+5);
-                    }
-                    temp = (char *)realloc(temp,len);
-                   if ( temp == NULL ){
-	              fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
-	              return NULL;
-                   }
-                    temp[0]='\0';
-                    strncpy(temp, "Provides:", 12);
-		    for(i = 0; i < pkg->provides_count; i++) {
-                        snprintf(provstr, LINE_LEN, "%s %s", i == 0 ? "" : ",", pkg->provides_str[i]);
-                        strncat(temp, provstr, strlen(provstr));           
-                    }
-                    strncat(temp, "\n", strlen("\n")); 
+               /* Here we check if the ipkg_internal_use_only is used, and we discard it.*/
+                  for ( i=0; i < pkg->provides_count; i++ ){
+	              if (strstr(pkg->provides_str[i],"ipkg_internal_use_only")!=NULL) {
+                         memset (pkg->provides_str[i],'\x0',strlen(pkg->provides_str[i])); /* Pigi clear my trick flag, just in case */
+                         flag_provide_false = 1;
+                      }
+                  }
+                  if ( !flag_provide_false ||                                             /* Pigi there is not my trick flag */
+                     ((flag_provide_false) &&  (pkg->provides_count > 1))){             /* Pigi There is, but we also have others Provides */
+                     char provstr[LINE_LEN];
+                     len = 15;
+		     for(i = 0; i < pkg->provides_count; i++) {
+                         len = len + (strlen(pkg->provides_str[i])+5);
+                     }
+                     temp = (char *)realloc(temp,len);
+                     if ( temp == NULL ){
+	                fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	                return NULL;
+                     }
+                     temp[0]='\0';
+                     strncpy(temp, "Provides:", 12);
+		     for(i = 0; i < pkg->provides_count; i++) {
+                         if (strlen(pkg->provides_str[i])>0){;
+                            snprintf(provstr, LINE_LEN, "%s %s", i == 1 ? "" : ",", pkg->provides_str[i]);
+                            strncat(temp, provstr, strlen(provstr));           
+                         }
+                     }
+                     strncat(temp, "\n", strlen("\n")); 
+                  }
                }
 	  } else {
 	       goto UNKNOWN_FMT_FIELD;
@@ -871,20 +884,25 @@ char * pkg_formatted_field(pkg_t *pkg, const char *field )
                }
 	  } else if (strcasecmp(field, "Status") == 0) {
 	       /* Status */
-               temp = (char *)realloc(temp,strlen(pkg_state_want_to_str(pkg->state_want))+
-                               strlen(pkg_state_flag_to_str(pkg->state_flag))+
-                               strlen(pkg_state_status_to_str(pkg->state_status))+12);
+               /* Benjamin Pineau note: we should avoid direct usage of 
+                * strlen(arg) without keeping "arg" for later free()
+                */
+               char *pflag=pkg_state_flag_to_str(pkg->state_flag);
+               char *pstat=pkg_state_status_to_str(pkg->state_status);
+               char *pwant=pkg_state_want_to_str(pkg->state_want);
+
+               size_t sum_of_sizes = (size_t) ( strlen(pwant)+ strlen(pflag)+ strlen(pstat) + 12 );
+               temp = (char *)realloc(temp,sum_of_sizes);
                if ( temp == NULL ){
-	          fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
-	          return NULL;
-               }
-               temp[0]='\0';
-               snprintf(temp, (strlen(pkg_state_want_to_str(pkg->state_want))+
-                               strlen(pkg_state_flag_to_str(pkg->state_flag))+
-                               strlen(pkg_state_status_to_str(pkg->state_status))+12), "Status: %s %s %s\n", 
-                                                             pkg_state_want_to_str(pkg->state_want),
-                                                             pkg_state_flag_to_str(pkg->state_flag),
-                                                             pkg_state_status_to_str(pkg->state_status));
+                   fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+                   return NULL;
+                }
+                temp[0]='\0';
+                snprintf(temp, sizeof(temp) , "Status: %s %s %s\n", pwant, pflag, pstat);
+                free(pflag);
+                free(pwant);
+               if(pstat) /* pfstat can be NULL if ENOMEM */
+                   free(pstat);
 	  } else if (strcasecmp(field, "Suggests") == 0) {
 	       if (pkg->suggests_count) {
 		    int i;
@@ -1140,7 +1158,7 @@ int pkg_name_version_and_architecture_compare(void *p1, void *p2)
      if (vercmp)
 	  return vercmp;
      if (!a->arch_priority || !b->arch_priority) {
-       fprintf(stderr, "pkg_name_version_and_architecture_compare: a=%p a->arch_priority=%p b=%p b->arch_priority=%p\n",
+       fprintf(stderr, "pkg_name_version_and_architecture_compare: a=%p a->arch_priority=%i b=%p b->arch_priority=%i\n",
 	       a, a->arch_priority, b, b->arch_priority);
        return 0;
      }
@@ -1450,13 +1468,13 @@ char *pkg_state_want_to_str(pkg_state_want_t sw)
 
      for (i=0; i < ARRAY_SIZE(pkg_state_want_map); i++) {
 	  if (pkg_state_want_map[i].value == sw) {
-	       return pkg_state_want_map[i].str;
+	       return strdup(pkg_state_want_map[i].str);
 	  }
      }
 
      fprintf(stderr, "%s: ERROR: Illegal value for state_want: %d\n",
 	     __FUNCTION__, sw);
-     return "<STATE_WANT_UNKNOWN>";
+     return strdup("<STATE_WANT_UNKNOWN>");
 }
 
 pkg_state_want_t pkg_state_want_from_str(char *str)
@@ -1541,13 +1559,13 @@ char *pkg_state_status_to_str(pkg_state_status_t ss)
 
      for (i=0; i < ARRAY_SIZE(pkg_state_status_map); i++) {
 	  if (pkg_state_status_map[i].value == ss) {
-	       return pkg_state_status_map[i].str;
+	       return strdup(pkg_state_status_map[i].str);
 	  }
      }
 
      fprintf(stderr, "%s: ERROR: Illegal value for state_status: %d\n",
 	     __FUNCTION__, ss);
-     return "<STATE_STATUS_UNKNOWN>";
+     return strdup("<STATE_STATUS_UNKNOWN>");
 }
 
 pkg_state_status_t pkg_state_status_from_str(char *str)
@@ -1687,14 +1705,14 @@ int pkg_write_filelist(ipkg_conf_t *conf, pkg_t *pkg)
 	  ipkg_message(conf, IPKG_ERROR, "Null pkg\n");
 	  return -EINVAL;
      }
-     ipkg_message(conf, IPKG_NOTICE,
+     ipkg_message(conf, IPKG_INFO,
 		  "    creating %s.list file\n", pkg->name);
      sprintf_alloc(&list_file_name, "%s/%s.list", pkg->dest->info_dir, pkg->name);
      if (!list_file_name) {
 	  ipkg_message(conf, IPKG_ERROR, "Failed to alloc list_file_name\n");
 	  return -ENOMEM;
      }
-     ipkg_message(conf, IPKG_NOTICE,
+     ipkg_message(conf, IPKG_INFO,
 		  "    creating %s file for pkg %s\n", list_file_name, pkg->name);
      data.stream = fopen(list_file_name, "w");
      if (!data.stream) {
@@ -1720,7 +1738,7 @@ int pkg_write_changed_filelists(ipkg_conf_t *conf)
      if (conf->noaction)
 	  return 0;
 
-     ipkg_message(conf, IPKG_NOTICE, "%s: saving changed filelists\n", __FUNCTION__);
+     ipkg_message(conf, IPKG_INFO, "%s: saving changed filelists\n", __FUNCTION__);
      pkg_hash_fetch_all_installed(pkg_hash, installed_pkgs);
      for (i = 0; i < installed_pkgs->len; i++) {
 	  pkg_t *pkg = installed_pkgs->pkgs[i];
